<?php

namespace Teknisa\EST\Services;

use Teknisa\Model\Geral\Entities\Paramfilial;
use Teknisa\Model\Geral\Entities\Paramgeral;
use \Doctrine\ORM\EntityManager;
use Teknisa\EST\Factories\InventarioRotativoFactory;
use Teknisa\Libs\Util\Environment;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class InventarioRotativoService {

    protected $entityManager;
    protected $environment;
    protected $inveRotaFactory;

    public function __construct (EntityManager $entityManager, Environment $environment, InventarioRotativoFactory $inveRotaFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->inveRotaFactory = $inveRotaFactory;
    }

    
    /**
     * Busca Rateio Tipo de Conta Pagar
     * @param $nrorg
     * @param $cdtpctpagar
     * @return Rattpctp
     */
    public function buscaParamFilial($nrorg, $cdfilial){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial'] = $cdfilial;

        /** @var Rattpctp $rattpctp */
        $ParamFilial = $this->entityManager->getRepository(Paramfilial::class)->findBy($params);

        return $ParamFilial;
    }
    
    public function buscaParamGeral($nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        
        /** @var Rattpctp $rattpctp */
        $ParamGeral = $this->entityManager->getRepository(Paramgeral::class)->findBy($params);

        return $ParamGeral;
    }
    
    public function validateInventarioRotativo($cdfilial = false) {
        if (!$cdfilial) {
            $cdfilial = $this->environment->getCdFilial();
        }
        $nrorg = $this->environment->getNrOrg();
        $paramFilial = $this->inveRotaFactory->getParamFilial($cdfilial, $nrorg);
        $paramGeral = $this->inveRotaFactory->getParamGeral($nrorg);

        if (!($paramFilial && $paramGeral)) {
            return true;
        }
        if ($paramGeral['IDUTILDTPROC'] == 'S') {
            $dtprocessa = $paramFilial['DTPROCESSA'];
        }
        else {
            $dtprocessa = date('d/m/Y');
        }
        $utilizalote = ($paramFilial['IDUTILLOTE'] == "S");
        $params = array(":P_CDFILIAL" => $cdfilial, ":P_DTINVEROTA" => $dtprocessa);
        if ($utilizalote) {
            $data = $this->inveRotaFactory->getInventarioComLote($params);
        }
        else {
            $data = $this->inveRotaFactory->getInventarioSemLote($params);
        }
        return empty($data);
    }
    
    public function validateBlindInventory($cdfilial = false) {
        
        if (!$cdfilial)  $cdfilial = $this->environment->getCdFilial();
        
        $nrorg = $this->environment->getNrOrg();
        $paramFilial = $this->inveRotaFactory->getParamFilial($cdfilial, $nrorg);
        $paramGeral = $this->inveRotaFactory->getParamGeral($nrorg);

        if (!($paramFilial && $paramGeral))  return true;
        
        $dtprocessa = $paramGeral['IDUTILDTPROC'] == 'S' ? $paramFilial['DTPROCESSA'] : date('d/m/Y');
        
        $params = [
            ":CDFILIAL" => $cdfilial,
            ":DTPROCESSA" => $dtprocessa
        ];
        $data = $this->inveRotaFactory->getValidaInventario($params);
        return empty($data);
    }
    
    public function getInventCegoInfo($useDtProcessa,$cdfilial) {
        
        if (!$cdfilial)  $cdfilial = $this->environment->getCdFilial();
        
        $nrorg = $this->environment->getNrOrg();
        $paramFilial = $this->inveRotaFactory->getParamFilial($cdfilial, $nrorg);
        $paramGeral = $this->inveRotaFactory->getParamGeral($nrorg);
        
        if($useDtProcessa)
            $dtprocessa = is_null($paramFilial['DTPROCESSA']) ? date('d/m/Y') : $paramFilial['DTPROCESSA'];
        else
            $dtprocessa = date('d/m/Y');
        
        $params = [
            ":CDFILIAL" => $cdfilial,
            ":DTCONTAGEM" => $dtprocessa
        ];
        return $this->inveRotaFactory->getInventCegoInfo($params);
    }
    
    public function getGroupOperatorInfo() {
        $cdoperador = $this->environment->getCdOperador();

        $params = [":CDOPERADOR" => $cdoperador];
        $result = $this->inveRotaFactory->getGroupOperatorInfo($params);
        
        $formatedResult = [];
        foreach($result as $type)
            $formatedResult[] = $type["IDAUTGRUPOP"];
        
        return $formatedResult;
    }
    
    public function checkOperatorGroups($operatorType) {
        $operatorGroups = $this->getGroupOperatorInfo();
        foreach($operatorType as $Type){
            if(in_array($Type, $operatorGroups)){
                return true;
            }
        }
    }
    
    public function checkInventCego($InventCegoInfo,$InventCegoType) {
        foreach($InventCegoInfo as $InventCego){
            if(in_array($InventCego['IDSITESTCEGO'], $InventCegoType)){
                return true;
            }
        }
    }

    public function executaProcedureAjuteZeraInve(
        $evolest, $cdfilimovi, $cdfilial, $cdoperador, $dataproc, $inopcao, $cdservfili, $cdtiporeti, $cdcentcust, $cdsetor, $retiestq, $cdalmoxarife, $cdlocalestoq
    ) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_INVENTARIO_ESTOQUE');
        $procedure->addParam(new Param('P_EVOLEST', Param::PARAM_INPUT, $evolest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIMOVI', Param::PARAM_INPUT, $cdfilimovi, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dataproc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INOPCAO', Param::PARAM_INPUT, $inopcao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $cdservfili, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPORETI', Param::PARAM_INPUT, $cdtiporeti, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCENTCUST', Param::PARAM_INPUT, $cdcentcust, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSETOR', Param::PARAM_INPUT, $cdsetor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETIESTQ', Param::PARAM_INPUT, $retiestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $cdalmoxarife, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $cdlocalestoq, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
}